svn commit: r237559 - stable/9/sys/amd64/amd64

Konstantin Belousov kib at FreeBSD.org
Mon Jun 25 09:41:48 UTC 2012


Author: kib
Date: Mon Jun 25 09:41:47 2012
New Revision: 237559
URL: http://svn.freebsd.org/changeset/base/237559

Log:
  MFC r237243:
  Adjust the fix in r236953, by not generating the signal manually, but
  performing the return to usermode using full return path.

Modified:
  stable/9/sys/amd64/amd64/trap.c
Directory Properties:
  stable/9/sys/   (props changed)

Modified: stable/9/sys/amd64/amd64/trap.c
==============================================================================
--- stable/9/sys/amd64/amd64/trap.c	Mon Jun 25 08:34:14 2012	(r237558)
+++ stable/9/sys/amd64/amd64/trap.c	Mon Jun 25 09:41:47 2012	(r237559)
@@ -982,16 +982,10 @@ amd64_syscall(struct thread *td, int tra
 	 * If the user-supplied value of %rip is not a canonical
 	 * address, then some CPUs will trigger a ring 0 #GP during
 	 * the sysret instruction.  However, the fault handler would
 	 * execute with the user's %gs and %rsp in ring 0 which would
 	 * not be safe.  Instead, preemptively kill the thread with a
 	 * SIGBUS.
 	 */
-	if (td->td_frame->tf_rip >= VM_MAXUSER_ADDRESS) {
-		ksiginfo_init_trap(&ksi);
-		ksi.ksi_signo = SIGBUS;
-		ksi.ksi_code = BUS_OBJERR;
-		ksi.ksi_trapno = T_PROTFLT;
-		ksi.ksi_addr = (void *)td->td_frame->tf_rip;
-		trapsignal(td, &ksi);
-	}
+	if (td->td_frame->tf_rip >= VM_MAXUSER_ADDRESS)
+		set_pcb_flags(td->td_pcb, PCB_FULL_IRET);
 }
@@ -982,16 +982,10 @@ amd64_syscall(struct thread *td, int tra
 	 * address, then some CPUs will trigger a ring 0 #GP during
 	 * the sysret instruction.  However, the fault handler would
-	 * execute with the user's %gs and %rsp in ring 0 which would
-	 * not be safe.  Instead, preemptively kill the thread with a
-	 * SIGBUS.
+	 * execute in ring 0 with the user's %gs and %rsp which would
+	 * not be safe.  Instead, use the full return path which
+	 * catches the problem safely.
 	 */


More information about the svn-src-stable-9 mailing list